
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'acceptance_helpers'
require 'shared/contexts'

resource 'Complaints' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  explanation "With complaint an user may report a learning object or collection with inappropriate content."

  before {
    12.times {
      create(:complaint)
      create(:complaint_reason)
      create(:learning_object)
    }
    LearningObject.reindex
  }
  
  let(:complaints) { Complaint.all }
  let(:complaint_reasons) { ComplaintReason.all }
  let(:learning_objects) { LearningObject.all }

  get '/v1/complaints' do
    include_context "authenticate_user_editor"

    parameter :limit, 'Limit of complaints'
    parameter :offset, 'Offset of complaints'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of complaints' do
      # active model serializer may render model associations in different order for collections (array of items), so we're verifing only returned ids
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Complaint.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end

  get '/v1/complaints/:id' do
    include_context "authenticate_user_editor"

    let(:id) { Complaint.first.id }

    example_request 'Get a complaint' do 
      expect(status).to eq(200)
    end
  end

  get '/v1/complaints/:id/show_related' do
    include_context "authenticate_user_editor"

    let(:id) { Complaint.first.id }

    example_request 'show all complaints whtih the same complainable as the infomed id' do 
      
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Complaint.where(complainable: Complaint.first.complainable, state: "complained").pluck(:id).sort)
      expect(status).to eq(200)
    end
  end


  post '/v1/complaints' do
    include_context 'authenticate_user'

    parameter :user_id, 'The id of user', scope: :complaint
    parameter :description, 'The description of the complaint', scope: :complaint
    parameter :complainable_id, 'The id of the complainable [LearningObject, Review, User, Collection]', scope: :complaint
    parameter :complainable_type, 'Represents a type of the complainable [LearningObject, Review, User, Collection]', scope: :complaint
    parameter :complaint_reason_id, 'Id of the reason of the complaint', scope: :complaint


    let(:user_id) { @user.id }
    let(:description) { Faker::Lorem::sentence }
    let(:complainable_id) { learning_objects.first.id }
    let(:complainable_type) { "LearningObject" }
    let(:complaint_reason_id) { complaint_reasons.first.id }
    let(:raw_post) { params.to_json }
  
    example_request 'Creating a complaint' do
      expect(status).to eq(201)
    end

  end

  post '/v1/complaints/:id/accept' do
    include_context "authenticate_user_editor"
    
    parameter :option

    let(:option) { "blocked" }
    let(:id) { Complaint.first.id }
    let(:raw_post) { params.to_json }

    example_request 'Accepting a complaint' do 
      expect(status).to eq(200)
    end
  end

  post '/v1/complaints/:id/reject' do
    include_context "authenticate_user_editor"
  
    let(:id) { Complaint.second.id }

    example_request 'Rejecting a complaint' do 
      expect(status).to eq(200)
    end
  end

  post '/v1/complaints/:id/treat_related' do
    include_context "authenticate_user_editor"

    parameter :complaints, 'group of complaints related to to one Object'
    parameter :option, 'Inform the final sentence to the related Object that the complaints are directed to.'

    let(:id) { Complaint.where(complainable_type: "User", state: "complained").first.complainable_id}
    let(:complaints){ [{id: Complaint.where(complainable_type: "User", state: "complained").first.id, state: "accept"}] }
    let(:option) { "blocked" }
    let(:raw_post) { params.to_json }

    example_request 'Treat all pendent complaints related to the complainable indicated by the id' do 
      expect(status).to eq(200)
    end
  end

  get '/v1/complaints' do
    include_context "authenticate_user_editor"

    parameter :limit, 'Limit of complaints'
    parameter :offset, 'Offset of complaints'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of complaints' do
      # active model serializer may render model associations in different order for collections (array of items), so we're verifing only returned ids
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Complaint.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end


end
